const isBuiltinType = (type) => {
    const builtInScalarNames = ["Int", "Float", "String", "Boolean", "ID"];
    const builtInEnumNames = ["__TypeKind", "__DirectiveLocation"];
    const builtInObjectNames = [
        "__Schema",
        "__Type",
        "__Field",
        "__InputValue",
        "__Directive",
        "__EnumValue",
    ];
    return ((type.kind === "SCALAR" && builtInScalarNames.indexOf(type.name) !== -1) ||
        (type.kind === "ENUM" && builtInEnumNames.indexOf(type.name) !== -1) ||
        (type.kind === "OBJECT" && builtInObjectNames.indexOf(type.name) !== -1));
};
function parseFieldType(field, enums) {
    if (field.kind === "NON_NULL") {
        const inner = parseFieldType(field.ofType, enums);
        // If array is set, then this NON_NULL applies to an inner LIST type
        if (inner.array) {
            return Object.assign(Object.assign({}, inner), { nullableArray: false });
        }
        return Object.assign(Object.assign({}, inner), { nullable: false });
    }
    else if (field.kind === "LIST") {
        return Object.assign(Object.assign({}, parseFieldType(field.ofType, enums)), { array: true });
    }
    if ("name" in field) {
        let name;
        let builtIn;
        let isEnum = false;
        switch (field.name) {
            case "Int":
            case "Float":
                builtIn = true;
                name = "number";
                break;
            case "String":
            case "ID":
                builtIn = true;
                name = "string";
                break;
            case "Boolean":
                builtIn = true;
                name = "boolean";
                break;
            default:
                name = field.name;
                builtIn = false;
                break;
        }
        if (enums.some((el) => el.name === name)) {
            isEnum = true;
        }
        return {
            type: name,
            builtIn,
            isEnum,
            nullable: true,
            nullableArray: true,
            array: false,
        };
    }
    return {};
}
function parseInputField(field, enums) {
    const data = parseFieldType(field.type, enums);
    return Object.assign({ name: field.name, description: field.description }, data);
}
export function parseSchema(introspection) {
    const gqlTypes = introspection.__schema.types.filter((type) => !isBuiltinType(type));
    const enums = [];
    const unions = [];
    const interfaces = [];
    const objects = [];
    const inputObjects = [];
    // Enum prepass
    for (const type of gqlTypes) {
        switch (type.kind) {
            case "ENUM":
                const values = [];
                for (const value of type.enumValues) {
                    values.push({
                        description: value.description,
                        name: value.name,
                        value: value.name,
                    });
                }
                values.sort((a, b) => a.name.localeCompare(b.name));
                enums.push({
                    description: type.description,
                    name: type.name,
                    values,
                });
                break;
        }
    }
    for (const type of gqlTypes) {
        switch (type.kind) {
            case "ENUM":
                break;
            case "OBJECT":
                objects.push({
                    isInput: false,
                    isInterface: false,
                    isObject: true,
                    name: type.name,
                    description: type.description,
                    extends: type.interfaces.map((el) => el.name),
                    possibleTypes: [type.name],
                    fields: type.fields.map((field) => {
                        const data = parseFieldType(field.type, enums);
                        return Object.assign({ name: field.name, description: field.description, args: field.args.length > 0
                                ? field.args.map((el) => parseInputField(el, enums))
                                : type.name === "Query" || type.name === "Mutation"
                                    ? []
                                    : null }, data);
                    }),
                });
                break;
            case "INPUT_OBJECT":
                inputObjects.push({
                    isInput: false,
                    isInterface: false,
                    isObject: true,
                    name: type.name,
                    description: type.description,
                    extends: [],
                    possibleTypes: [type.name],
                    fields: type.inputFields.map((field) => {
                        return Object.assign(Object.assign({}, parseInputField(field, enums)), { args: null });
                    }),
                });
                break;
            case "INTERFACE":
                interfaces.push({
                    isInput: false,
                    isInterface: true,
                    isObject: false,
                    name: type.name,
                    description: type.description,
                    extends: [],
                    possibleTypes: [...type.possibleTypes.map((el) => el.name)],
                    fields: type.fields.map((field) => {
                        const data = parseFieldType(field.type, enums);
                        return Object.assign({ name: field.name, description: field.description, args: field.args.length > 0
                                ? field.args.map((el) => parseInputField(el, enums))
                                : type.name === "Query" || type.name === "Mutation"
                                    ? []
                                    : null }, data);
                    }),
                });
                break;
            case "UNION":
                unions.push({
                    name: type.name,
                    description: type.description,
                    unionOf: type.possibleTypes.map((el) => el.name),
                });
                break;
            default:
                throw new Error(`Unsupported kind: ${type.kind}`);
        }
    }
    enums.sort((a, b) => a.name.localeCompare(b.name));
    return {
        enums,
        unions,
        interfaces,
        objects,
        inputObjects,
    };
}
//# sourceMappingURL=parse.js.map